#include <xc.h>

void pwmInit(unsigned int resolution, unsigned int clockdiv)
{
    // only using PWM2L here, so module-specific config is done for module 2
    
    // first part of the initialisation has to happen with the PWM module off
    PTCONbits.PTEN = 0; // disable PWM module
    
    // after a reset, hardware fault / overcurrent handling is on and always
    // triggered (WHY?!?), so it has to be switched off:
    __builtin_write_PWMSFR(&FCLCON2, 0x0003, &PWMKEY);
    
    // configure the I/O-pins: PWM2L is a PWM pin, the PWM2 pair is in
    // complementary output mode, output to the pins is swapped
    // so PWM2L will actually output PWM2H
    __builtin_write_PWMSFR(&IOCON2, 0x4002, &PWMKEY);
    
    // as soon as the PWM module is on, it will put PWM1L and PWM3L in
    // fault mode and drive them low, too. That's stupid, so we have to
    // switch them off:
    __builtin_write_PWMSFR(&IOCON1, 0, &PWMKEY);
    __builtin_write_PWMSFR(&IOCON3, 0, &PWMKEY);
    __builtin_write_PWMSFR(&FCLCON1, 0x0003, &PWMKEY);
    __builtin_write_PWMSFR(&FCLCON3, 0x0003, &PWMKEY);
    
    PWMCON2 = 0; // most of PWMCON2 needs to be 0
    PWMCON2bits.DTC = 2; // switch off dead-time control, the DRV8801 does that
    
    // set the PWM period (and thus resolution), can be changed while PWM is on!
    PTPER = resolution;
    
    // set clock divider for the PWM module
    PTCON2bits.PCLKDIV = clockdiv; // divides by 2^(the value here + 1)
    // maximum value is 7 -> divide by 64
    // not changeable when the module is running!
    
    // last, switch on the PWM module
    PDC2 = 0; // zero the PWM output for now
    PTCONbits.PTEN = 1; // switch on the PWM module
}

void pwmSetDutyCycle(unsigned int dutycycle)
{
    PDC2 = dutycycle;
}
