#include <xc.h>
#include "qei.h"
#include "sw_config.h"

void qeiInit(void)
{
    QEI1IOCbits.HOMPOL = 1; // home signal is active low
    // zero out the position counter
    POS1HLD = 0;
    POS1CNTL = 0;
    QEI1CONbits.QEIEN = 1; // enable QEI module
}

void qeiSetHomingMode(void)
{
    QEI1ICH = 0;
    QEI1ICL = 0;
    QEI1CONbits.PIMOD = 3; // index when home is active resets position counter
}

void qeiSetSimpleHomingMode(void)
{
    QEI1ICH = 0;
    QEI1ICL = 0;
    QEI1CONbits.PIMOD = 2; // index resets position counter
}

void qeiSetNormalMode(void)
{
    QEI1CONbits.PIMOD = 0; // index input is counted, but has no effect
}

long qeiReadPosition(void)
{
    union
    {
        long pos_all;
        struct
        {
            int pos_low;
            int pos_high;
        };
    } position;
    
    position.pos_low = POS1CNTL; // read low word first, this stores the high
    // word in the hold register:
    position.pos_high = POS1HLD; // reads the upper word
    return position.pos_all;
}

int qeiReadSpeed(void)
{
    return VEL1CNT;
}
